<?php
require_once 'abstract.ilPluginControllerGUI.php';
/**
 * Class ilQuestionSetPoolPathGraphGUI
 */
class ilQuestionSetPoolPathGraphGUI extends  ilPluginControllerGUI
{
	/**
	 * @param ilObjQuestionSetPoolGUI $controller
	 */
	public function __construct(ilObjQuestionSetPoolGUI $controller)
	{
		parent::__construct($controller);
		$this->addSubTabs();
	}

	/**
	 * 
	 */
	public function showPathOverview()
	{
		$set = new ilQuestionSetPoolSet((int)$_GET['set_id']);
		$set->read();

		$template = new ilTemplate('tpl.il_xqsp_path_overview.html', true, true, 'Customizing/global/plugins/Services/Repository/RepositoryObject/QuestionSetPool');

		$this->renderPathTable($set, $template);
		$this->renderPathGraph($set, $template);

		$template->setVariable('SET_TITLE', $set->getTitle());
		$template->setVariable('GRAPH_TITLE', $this->plugin->txt("graph_viz_title"));

		$this->tpl->setContent($template->get());
	}

	/**
	 * 
	 */
	public function renderPathGraph($set, $template)
	{
		$renderedNodes = [];

		$pathes = $set->getPathList()->getData();
		foreach($pathes as $path)
		{
			$model = new ilQuestionSetPoolPath($path['id']);
			$model->read();
			$nodes = $model->getNodes()->getData();
			$num_nodes = count($nodes);
			for($i = 0; $i < $num_nodes; $i++)
			{
				if(!in_array($nodes[$i]['question_index'], $renderedNodes))
				{
					$template->setCurrentBlock('nodes');
					$template->setVariable('NODE_NAME', 'Q' . $nodes[$i]['question_index']);
					$template->setVariable('NODE_ID', 'Q' . $nodes[$i]['question_index']);
					$template->parseCurrentBlock();

					$renderedNodes[] = $nodes[$i]['question_index'];
				}

				$renderedNodes[] = $nodes[$i]['question_index'];
			}
		}

		$edges = array();

		foreach($pathes as $path)
		{
			$model = new ilQuestionSetPoolPath($path['id']);
			$model->read();
			$nodes = $model->getNodes()->getData();
			$num_nodes = count($nodes);
			for($i = 0; $i < $num_nodes; $i++)
			{
				if(isset($nodes[$i + 1]))
				{
					$edges["Q".$nodes[$i]['question_index']]["Q".$nodes[$i + 1]['question_index']]['initial'][trim($nodes[$i]['initial_condition'])] = $nodes[$i]['initial_condition'];
				}
				else
				{
					$edges["Q".$nodes[$i]['question_index']][$this->controller->plugin->txt('end_of_set')]['initial'][trim($nodes[$i]['initial_condition'])] = $nodes[$i]['initial_condition'];
				}

				foreach((array)$nodes[$i]['jump_conditions'] as $condition)
				{
					if($condition['question_index'])
					{
						$edges["Q".$nodes[$i]['question_index']]["Q".$condition['question_index']]['jump'][trim($condition['jump_condition'])] = $condition['jump_condition'];
					}
					else
					{
						$edges["Q".$nodes[$i]['question_index']][$this->controller->plugin->txt('end_of_set')]['end'][trim($condition['jump_condition'])] = $condition['jump_condition'];
					}
				}
			}
		}

		$i = 0;
		foreach($edges as $node_source => $data_target)
		{
			foreach($data_target as $node_target => $data_conditions)
			{
				++$i;
				if(count($data_conditions['initial']) > 0)
				{
					$initial = join("|", array_filter($data_conditions['initial']));

					$template->setCurrentBlock('initial');
					$template->setVariable('INITIAL_ID', 'iidx' . $i . $initial);
					$template->setVariable('INITIAL_NAME', $initial);
					$template->setVariable('INITIAL_SOURCE', $node_source);
					$template->setVariable('INITIAL_TARGET', $node_target);
					$template->parseCurrentBlock();

				}
				if(count($data_conditions['jump']) > 0)
				{
					$jump = join("|", array_filter($data_conditions['jump']));

					$template->setCurrentBlock('jump');
					$template->setVariable('JUMP_ID', 'jidx' . $i . $jump);
					$template->setVariable('JUMP_NAME', $jump);
					$template->setVariable('JUMP_SOURCE', $node_source);
					$template->setVariable('JUMP_TARGET', $node_target);
					$template->parseCurrentBlock();

				}
				if(count($data_conditions['end']) > 0)
				{
					$end = join("|", array_filter($data_conditions['end']));

					$template->setCurrentBlock('jump');
					$template->setVariable('END_ID', 'eidx' . $i . $end);
					$template->setVariable('END_NAME', $end);
					$template->setVariable('END_SOURCE', $node_source);
					$template->setVariable('END_TARGET', $node_target);
					$template->parseCurrentBlock();

				}
			}
		}

		$template->setCurrentBlock('end_node');
		$template->setVariable('END_NODE_NAME', $this->controller->plugin->txt('end_of_set'));
		$template->parseCurrentBlock();
	}
	
	/**
	 * Returns the current tab name
	 * return string
	 */
	protected function getCurrentTab()
	{
		return 'content';
	}

	/**
	 * @return array
	 */
	protected function getRequirements()
	{
		return array(
			'models/class.ilQuestionSetPoolSet.php',
			'models/class.ilQuestionSetPoolPath.php'
		);
	}

	private function addSubTabs()
	{
		/** @var ilTabsGUI $ilTabs */
		global $ilTabs;

		$this->ctrl->setParameter($this->controller, "set_id", (int)$_GET['set_id']);

		$ilTabs->addSubTab("set_questions", $this->plugin->txt("questions"),
			$this->ctrl->getLinkTarget($this->controller, "ilQuestionSetPoolSetQuestionsGUI.listQuestions"));

		$ilTabs->addSubTab("paths", $this->plugin->txt("paths"),
			$this->ctrl->getLinkTarget($this->controller, "ilQuestionSetPoolPathsGUI.listPaths"));

		$ilTabs->addSubTab("path_overview", $this->plugin->txt("path_overview"),
			$this->ctrl->getLinkTarget($this->controller, "ilQuestionSetPoolPathGraphGUI.showPathOverview"));

		$ilTabs->activateSubTab("path_overview");

		$this->ctrl->setParameter($this->controller, "set_id", "");
	}

	/**
	 * @param $set
	 * @param $template
	 */
	protected function renderPathTable($set, $template)
	{
		$pathes = $set->getPathList()->getData();
		foreach ($pathes as $path_data)
		{
			$path = new ilQuestionSetPoolPath($path_data['id']);
			$path->read();
			$nodes = $path->getNodes()->getData();

			foreach ($nodes as $node)
			{
				$template->setCurrentBlock("initial_condition");
				$condition = $node["initial_condition"];
				if ($condition == "")
				{
					$condition = $this->plugin->txt("empty_condition");
				}
				$template->setVariable("INITIAL_CONDITION", $condition);
				$template->parseCurrentBlock();

				$template->setCurrentBlock("question");
				$template->setVariable("QUESTION_INDEX", $node["question_index"]);
				$template->parseCurrentBlock();

				if (count($node["jump_conditions"]) > 0)
				{
					foreach ($node["jump_conditions"] as $jump_condition)
					{
						$condition = "";

						if ($jump_condition["question_index"] != null)
						{
							$condition .= $this->plugin->txt("target_question") . ": ";
							$condition .= "Q" . $jump_condition["question_index"];
						}
						else
						{
							$condition .= $this->plugin->txt("tst_termination");
						}
						$template->setCurrentBlock("jump_condition");
						$template->setVariable("JUMP_CONDITION", $jump_condition["jump_condition"]);
						$template->setVariable("TARGET", $condition . "<br />". $this->plugin->txt("repititions") . ": " . $jump_condition["repititions"]);
						$template->parseCurrentBlock();
					}
				}
				else
				{
					$template->setCurrentBlock("jump_condition");
					$template->setVariable("JUMP_CONDITION", "&nbsp;");
					$template->setVariable("TARGET", "&nbsp;");
					$template->parseCurrentBlock();
				}
				$template->setCurrentBlock("jump_conditions");
				$template->parseCurrentBlock();
			}

			$template->setCurrentBlock("test_end");
			$template->setVariable("END_TEST", $this->plugin->txt('end_of_test'));
			$template->parseCurrentBlock();

			$template->setCurrentBlock("path");
			$template->setVariable("PATH_TITLE", $path->getTitle());
			$template->parseCurrentBlock();
		}
	}
} 